/*
 * Copyright (C) 2013-2015 Apple Inc. All rights reserved.
 *
 * This document is the property of Apple Inc.
 * It is considered confidential and proprietary.
 *
 * This document may not be reproduced or transmitted in any form,
 * in whole or in part, without the express written permission of
 * Apple Inc.
 */
#ifndef __APPLE_DISPLAY_TIMINGS_H
#define __APPLE_DISPLAY_TIMINGS_H

//Supported timings by this platform
static struct display_timing timing_list[] =
{
#if DEBUG_BUILD || SUB_TARGET_K93 || SUB_TARGET_K94 || SUB_TARGET_K95 || SUB_TARGET_K93A
	{ 
	.display_name = 	"k48",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		68400000, 
	.dot_pitch = 		132, 
	.h_active =		1024,
	.h_pulse_width = 	135, 
	.h_back_porch = 	133,
	.h_front_porch =        133,
	.v_active = 		768, 
	.v_pulse_width =  	12, 
	.v_back_porch = 	10, 
	.v_front_porch =        10, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	18, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
#endif
#if DEBUG_BUILD || SUB_TARGET_N94
	{ 
	.display_name = 	"n94",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		51300000, 
	.dot_pitch = 		326,
	.h_active =		640, 
	.h_pulse_width =	73, 
	.h_back_porch =		71, 
	.h_front_porch =       	71, 
	.v_active = 		960, 
	.v_pulse_width =  	16, 
	.v_back_porch = 	12, 
	.v_front_porch =       	12, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	24, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
	
	{ 
	.display_name = 	"n94-2",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		51300000, 
	.dot_pitch = 		326, 
	.h_active =		640, 
	.h_pulse_width = 	73, 
	.h_back_porch = 	71, 
	.h_front_porch =       	71, 
	.v_active = 		960,
	.v_pulse_width =  	16, 
	.v_back_porch = 	12, 
	.v_front_porch =       	12, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	24, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
	
	{ 
	.display_name = 	"d300",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		60350000, 
	.dot_pitch = 		326, 
	.h_active =		640,
	.h_pulse_width = 	48, 
	.h_back_porch = 	48, 
	.h_front_porch =       	48, 
	.v_active = 		960,
	.v_pulse_width =	3,  
	.v_back_porch = 	13, 
	.v_front_porch =       	307, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	24, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
#endif
#if DEBUG_BUILD || SUB_TARGET_P105 || SUB_TARGET_P106 || SUB_TARGET_P107
	{ 
	.display_name = 	"ipad2b",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		68400000, 
	.dot_pitch = 		163, 
	.h_active =		768,
	.h_pulse_width = 	80, 
	.h_back_porch = 	76,
	.h_front_porch =        76,
	.v_active = 		1024,
	.v_pulse_width =  	40, 
	.v_back_porch = 	38, 
	.v_front_porch =        38,
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	18, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
	
	{ 
	.display_name = 	"ipad2b-p0",
	.host_clock_id =	CLK_VCLK0, 
	.pixel_clock = 		68400000, 
	.dot_pitch = 		163, 
	.h_active =		768,
	.h_pulse_width = 	80, 
	.h_back_porch = 	113,
	.h_front_porch =       	113,
	.v_active = 		1024,
	.v_pulse_width =  	18, 
	.v_back_porch = 	10, 
	.v_front_porch =       	10, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	18, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
#endif
#if DEBUG_BUILD || WITH_HW_DISPLAY_DISPLAYPORT
	{ 
	.display_name = 	"720p",	
	.host_clock_id = 	CLK_VCLK1, 
	.pixel_clock = 		74250000,
	.dot_pitch = 		243, 
	.h_active =		1280,
	.h_pulse_width = 	40,  
	.h_back_porch = 	220, 
	.h_front_porch =        110, 
	.v_active = 		720,
	.v_pulse_width =  	5, 
	.v_back_porch = 	20, 
	.v_front_porch =        5, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	24, 
	.display_type = 	DISPLAY_TYPE_DP,     
	.display_config		= NULL,
	},
#endif
#if DEBUG_BUILD || SUB_TARGET_N78
	{ 
	.display_name = 	"n41",	
	.host_clock_id = 	CLK_VCLK0, 
	.pixel_clock = 		68400000, 
	.dot_pitch = 		326, 
	.h_active =		640,
	.h_pulse_width = 	40,  
	.h_back_porch = 	40, 
	.h_front_porch =       	40,
	.v_active = 		1136,
	.v_pulse_width =  	3,  
	.v_back_porch = 	13, 
	.v_front_porch =       	348, 
	.neg_vclk = 		0, 
	.neg_hsync = 		0, 
	.neg_vsync = 		0, 
	.neg_vden = 		0, 
	.display_depth = 	24, 
	.display_type = 	DISPLAY_TYPE_PINOT, 
	.display_config		= NULL,
	},
#endif
#if DEBUG_BUILD || WITH_HW_DISPLAY_HDMI
	{ 
	.display_name =   	"720p-hdmi",	
	.host_clock_id =  	CLK_VCLK1, 
	.pixel_clock = 	  	74250000, 
	.dot_pitch = 	  	243, 
	.h_active =	  	1280,
	.h_pulse_width =  	40, 
	.h_back_porch =   	220, 
	.h_front_porch =  	110, 
	.v_active = 	  	720,
	.v_pulse_width =  	5,  
	.v_back_porch =   	20, 
	.v_front_porch =  	5,
	.neg_vclk = 	  	0, 
	.neg_hsync = 	  	0, 
	.neg_vsync = 	 	0, 
	.neg_vden = 	  	0, 
	.display_depth =  	24, 
	.display_type =   	DISPLAY_TYPE_HDMI,     
	.display_config		= NULL,
	},
#endif
};
#endif	/* ! __APPLE_DISPLAY_TIMINGS_H */
